tuple_hash

Struct TupleHashXof

pub struct TupleHashXof<X> { /* private fields */ }
A cryptographic hash over a set of strings such that each string is unambiguously encoded.

For example, the TupleHash of ("abc", "d") will produce a different hash value than the TupleHash of ("ab", "cd").

Warning

TupleHash is only defined for cSHAKE128 and cSHAKE256. Using this with a different XOF might have worse security properties.

Implementations

Source

impl<X: Xof> TupleHashXof<X>

pub fn new(s: &[u8]) -> Self

Creates a TupleHash with the customization string s.

pub fn update(&mut self, s: &[u8])

Writes the string s to the hash.

pub fn finalize_xof(self) -> X::Reader

Returns a variable-size output.